@@ -1,6 +1,6 @@
 /*
  * XML LDAP client
- * $Id: xdap.c,v 1.9.2.1 2003/09/01 11:50:20 dsandras Exp $
+ * $Id: xdap.c,v 1.18 2004/05/26 20:55:51 dsandras Exp $
  * Copyright 2001 paul666@mailandnews.com
  * Licensed under version 2 or later of the GNU GPL
  */
@@ -11,9 +11,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "../../config.h"
 
 #include "xdap.h"
 
+#ifndef WIN32
+#define stricmp	strcasecmp
+#endif
+
 static int doldapop (LDAP *, xmlDocPtr, int, xmlNodePtr, int, char *, int,
 		     char *, char **, int);
 static int addattr (LDAPMod **, int *, char *, char *, int);
@@ -41,11 +46,13 @@ xdapfree(void)
 	int i;
 	for (i = 0; i < nregs; i++)
 		if (regs[i].refcount != 0) {
-			if ((regs[i].val == 0) || (regs[i].val == (void *)0xdeadbeef))
-				fprintf(stderr, "BOGUSFREE %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
+			if ((regs[i].val == 0) || (regs[i].val == (void *)0xdeadbeef)) {
+				////fprintf(stderr, "BOGUSFREE %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
+                                //
+                        }
 			else {
 #if DEBUGXDAPLEAK
-				fprintf(stderr, "FREE %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
+				//fprintf(stderr, "FREE %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
 #endif
 				free(regs[i].val);
 				regs[i].refcount = 0;
@@ -89,13 +96,11 @@ registerptr(void *ptr)
 		}
 		regs[i].refcount = 1;
 		regs[i].val = ptr;
-		if (i >= REGMAX - 3) /* overflow */
-			fprintf(stderr, "OVERFLOW %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
-		else
-			nregs++;
+		if (i < REGMAX - 3) 
+                  nregs++;
 	}
 #if DEBUGXDAPLEAK
-	fprintf(stderr, "RP %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
+	//fprintf(stderr, "RP %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
 #endif
 }
 
@@ -118,7 +123,7 @@ deregisterNode(xmlNodePtr node)
 	for (i = 0; i < nregs; i++)
 		if ((regs[i].refcount > 0) && (regs[i].val == node->_private)) {
 			regs[i].refcount--;
-			fprintf(stderr, "DN %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
+			//fprintf(stderr, "DN %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
 			if (regs[i].refcount == 0) {
 				free(regs[i].val);
 				regs[i].val = (void *)0xdeadbeef;
@@ -129,7 +134,7 @@ deregisterNode(xmlNodePtr node)
 		regs[i].refcount = -1;
 		regs[i].val = node->_private;
 #if DEBUGXDAPLEAK
-		fprintf(stderr, "DN %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
+	//fprintf(stderr, "DN %d %d %lx\n", i, regs[i].refcount, (unsigned long)regs[i].val);
 #endif
 	}
 }
@@ -156,7 +161,12 @@ parseonly (char *filename, xmlEntityPtr (*getent) (void *, const xmlChar *),
 	   xmlEntityPtr (**oldgetent) (void *, const xmlChar *), int noerr)
 {
   xmlDocPtr xp;
+
+#if HAVE_XMLSAXHANDLERV1
+  xmlSAXHandlerV1 sax;
+#else
   xmlSAXHandler sax;
+#endif
 
   memset (&sax, 0, sizeof sax);
   sax = xmlDefaultSAXHandler;	/* Copy original handler */
@@ -171,7 +181,6 @@ parseonly (char *filename, xmlEntityPtr (*getent) (void *, const xmlChar *),
   xmlRegisterNodeDefault(registerNode);
   xmlDeregisterNodeDefault(deregisterNode);
 #endif
-  xmlInitParser ();
 
   if (!(xp = xmlSAXParseFile (&sax, filename, 0)))
     return 0;
@@ -259,17 +268,17 @@ getldapinfo (xmlDocPtr xp, xmlNodePtr * current, char **hostp, int *portp,
   /* Ignore root element name */
   if (!(method = xmlGetProp (root, BAD_CAST "method")))
     method = xmlStrdup (BAD_CAST "SIMPLE");
-  if (!strcasecmp ((char *) method, "NONE"))
+  if (!stricmp ((char *) method, "NONE"))
     imethod = LDAP_AUTH_NONE;
-  else if (!strcasecmp ((char *) method, "SIMPLE"))
+  else if (!stricmp ((char *) method, "SIMPLE"))
     imethod = LDAP_AUTH_SIMPLE;
-  else if (!strcasecmp ((char *) method, "SASL"))
+  else if (!stricmp ((char *) method, "SASL"))
     imethod = LDAP_AUTH_SASL;
-  else if (!strcasecmp ((char *) method, "KRBV4"))
+  else if (!stricmp ((char *) method, "KRBV4"))
     imethod = LDAP_AUTH_KRBV4;
-  else if (!strcasecmp ((char *) method, "KRBV41"))
+  else if (!stricmp ((char *) method, "KRBV41"))
     imethod = LDAP_AUTH_KRBV41;
-  else if (!strcasecmp ((char *) method, "KRBV42"))
+  else if (!stricmp ((char *) method, "KRBV42"))
     imethod = LDAP_AUTH_KRBV42;
   else {
     xmlFree (method);
@@ -341,7 +350,7 @@ ldaprun (LDAP * ldap, xmlDocPtr xp, xmlNodePtr * current, int *opp,
   int optype;		/* ldap operation */
   xmlChar *base = 0;	/* search base */
   xmlChar *scope = 0;	/* search scope symbolic */
-  int iscope;		/* search scope */
+  int iscope = 0;	/* search scope */
   xmlChar *filter = 0;	/* search filter */
   xmlChar *attrlist = 0;	/* attribute list to return */
   char *attra[MAXATTRS];	/* attribute list in array form */
@@ -475,11 +484,11 @@ ldaprun (LDAP * ldap, xmlDocPtr xp, xmlNodePtr * current, int *opp,
       filter = xmlStrdup (BAD_CAST "(objectClass=*)");
     if (!(attrlist = xmlGetProp (*current, BAD_CAST "attrs")))
       attrlist = 0;
-    if (!strcasecmp ((char *) scope, "SUBTREE"))
+    if (!stricmp ((char *) scope, "SUBTREE"))
       iscope = LDAP_SCOPE_SUBTREE;
-    else if (!strcasecmp ((char *) scope, "BASE"))
+    else if (!stricmp ((char *) scope, "BASE"))
       iscope = LDAP_SCOPE_BASE;
-    else if (!strcasecmp ((char *) scope, "ONELEVEL"))
+    else if (!stricmp ((char *) scope, "ONELEVEL"))
       iscope = LDAP_SCOPE_ONELEVEL;
     else {
       xmlFree (scope);
@@ -562,7 +571,7 @@ doldapop (LDAP * ldap, xmlDocPtr xp, int sync, xmlNodePtr node, int optype,
   LDAPMod *attrs[MAXATTRS];	/* attribute pointers */
   LDAPMessage *res;	/* result list from search */
   int nattrs = 0;		/* Number of attributes */
-  int rc;
+  int rc = 0;
   char *text = 0;		/* shell command */
   xmlChar *modop;		/* attribute mod_op */
   xmlChar *eval;		/* evaluate the element text */
