@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  *                         lid.cpp  -  description
  *                         -----------------------
  *   begin                : Sun Dec 1 2002
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains LID functions.
  *
  */
@@ -43,35 +43,15 @@
 #include "gnomemeeting.h"
 #include "urlhandler.h"
 #include "misc.h"
+#include "log_window.h"
 #include "main_window.h"
 #include "pref_window.h"
 #include "callbacks.h"
 
 #include "dialog.h"
-#include "gconf_widgets_extensions.h"
+#include "gm_conf.h"
 
 
-static gboolean transfer_callback_cb (gpointer data);
-
-/* Declarations */
-extern GtkWidget *gm;
-
-
-/* DESCRIPTION  :  /
- * BEHAVIOR     :  Hack to be able to call the transfer_call_cb in as a g_idle
- *                 from a thread.
- * PRE          :  /
- */
-static gboolean
-transfer_callback_cb (gpointer data)
-{
-  gdk_threads_enter ();
-  transfer_call_cb (NULL, NULL);
-  gdk_threads_leave ();
-
-  return false;
-}
-
 
 #ifdef HAS_IXJ
 GMLid::GMLid (PString d)
@@ -81,10 +61,9 @@ GMLid::GMLid (PString d)
   soft_codecs = FALSE;
   
   dev_name = d;
-
-  /* Open the device */
+  
   Open ();
-
+  
   this->Resume ();
   thread_sync_point.Wait ();
 } 
@@ -94,19 +73,20 @@ GMLid::~GMLid ()
 {
   stop = TRUE;
   
-  PWaitAndSignal m(quit_mutex);
-  
   Close ();
+
+  PWaitAndSignal m(quit_mutex);
 }
 
 
 BOOL
 GMLid::Open ()
 {
+  GtkWidget *main_window = NULL;
+  GtkWidget *history_window = NULL;
+  
   GMH323EndPoint *ep = NULL;
-  GmWindow *gw = NULL;
 
-  gchar *msg = NULL;
   gchar *lid_country = NULL;
 
   int lid_aec = 0;
@@ -116,28 +96,23 @@ GMLid::Open ()
 
   PWaitAndSignal m(device_access_mutex);
   
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
   ep = GnomeMeeting::Process ()->Endpoint ();
   
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+  history_window = GnomeMeeting::Process ()->GetHistoryWindow ();
+  
   if (!IsOpen ()) {
 
     /* We use the default settings, but the device name provided as argument */
-    gnomemeeting_threads_enter ();
-    lid_country = gconf_get_string (AUDIO_DEVICES_KEY "lid_country_code");
-    lid_aec = gconf_get_int (AUDIO_DEVICES_KEY "lid_echo_cancellation_level");
-    lid_odt = gconf_get_int (AUDIO_DEVICES_KEY "lid_output_device_type");
-    gnomemeeting_threads_leave ();
+    lid_country = gm_conf_get_string (AUDIO_DEVICES_KEY "lid_country_code");
+    lid_aec = gm_conf_get_int (AUDIO_DEVICES_KEY "lid_echo_cancellation_level");
+    lid_odt = gm_conf_get_int (AUDIO_DEVICES_KEY "lid_output_device_type");
 
     if (OpalIxJDevice::Open (dev_name)) {
-      
-      msg = g_strdup_printf (_("Opened Quicknet device %s"), 
-			     (const char *) GetName ());
-      
-      gnomemeeting_threads_enter ();
-      gnomemeeting_log_insert (gw->history_text_view, msg);
-      gnomemeeting_threads_leave ();
 
-      g_free (msg);
+      gm_history_window_insert (history_window,
+				_("Opened Quicknet device %s"), 
+				(const char *) GetName ()); 
       
       if (lid_country)
 	SetCountryCodeName(lid_country);
@@ -157,9 +132,7 @@ GMLid::Open ()
     }
     else {
       
-      gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Error while opening the Quicknet device."), _("Please check that your driver is correctly installed and that the device is working correctly."));
-      gnomemeeting_threads_leave ();
+      gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Error while opening the Quicknet device."), _("Please check that your driver is correctly installed and that the device is working correctly."));
 
       return_val = FALSE;
     }
@@ -174,28 +147,22 @@ GMLid::Open ()
 BOOL
 GMLid::Close ()
 {
-  GmWindow *gw = NULL;
-  gchar *msg = NULL;
+  GtkWidget *history_window = NULL;
   
   PWaitAndSignal m(device_access_mutex);
 
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  history_window = GnomeMeeting::Process ()->GetHistoryWindow ();
+  
   
   if (IsOpen ()) {
 
-    msg =
-      g_strdup_printf (_("Closed Quicknet device %s"), 
-		       (const char *) GetName ());
-
     OpalIxJDevice::Close ();
 
-    gnomemeeting_threads_enter ();
-    gnomemeeting_log_insert (gw->history_text_view, msg);
-    gnomemeeting_threads_leave ();
+    gm_history_window_insert (history_window,
+			      _("Closed Quicknet device %s"), 
+			      (const char *) GetName ());
   }
 
-  g_free (msg);
-
   return TRUE;
 }
 
@@ -203,10 +170,11 @@ GMLid::Close ()
 void
 GMLid::Main ()
 {
-  GMH323EndPoint *endpoint = NULL;
+  GtkWidget *main_window = NULL;
+  GtkWidget *history_window = NULL;
+  GtkWidget *prefs_window = NULL;
 
-  GmWindow *gw = NULL;
-  GmPrefWindow *pw = NULL;
+  GMH323EndPoint *endpoint = NULL;
 
   BOOL off_hook = TRUE;
   BOOL last_off_hook = TRUE;
@@ -219,41 +187,44 @@ GMLid::Main ()
   PTime now;
   PTime last_key_press;
 
-  unsigned int vol = 0;
+  PString call_token;
+
+  BOOL is_on_hold = FALSE;
+
+  unsigned int input_vol = 0;
+  unsigned int output_vol = 0;
   int lid_odt = 0;
   GMH323EndPoint::CallingState calling_state = GMH323EndPoint::Standby;
-  
+
+
   PWaitAndSignal m(quit_mutex);
   thread_sync_point.Signal ();
 
   
   endpoint = GnomeMeeting::Process ()->Endpoint ();
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
-  pw = GnomeMeeting::Process ()->GetPrefWindow ();
 
-  
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+  history_window = GnomeMeeting::Process ()->GetHistoryWindow ();
+  prefs_window = GnomeMeeting::Process ()->GetPrefsWindow ();
+
+
   /* Check the initial hook status. */
   off_hook = last_off_hook = IsLineOffHook (OpalIxJDevice::POTSLine);
 
   gnomemeeting_threads_enter ();
-  lid_odt = gconf_get_int (AUDIO_DEVICES_KEY "lid_output_device_type");
+  lid_odt = gm_conf_get_int (AUDIO_DEVICES_KEY "lid_output_device_type");
   gnomemeeting_threads_leave ();
   
 
-  gnomemeeting_threads_enter ();
   /* Update the mixers if the lid is used */
-  GetPlayVolume (0, vol);
-  GTK_ADJUSTMENT (gw->adj_play)->value = (int) (vol);
-  GetRecordVolume (0, vol);
-  GTK_ADJUSTMENT (gw->adj_rec)->value = (int) (vol);
-  gtk_widget_queue_draw (GTK_WIDGET (gw->audio_settings_frame));
-
-  /* Update the codecs list */
-  gnomemeeting_codecs_list_build (pw->codecs_list_store, TRUE, soft_codecs);
+  GetPlayVolume (0, output_vol);
+  GetRecordVolume (0, input_vol);
+  
+    
+  gnomemeeting_threads_enter ();
+  gm_main_window_set_volume_sliders_values (main_window, output_vol, input_vol);
   gnomemeeting_threads_leave ();
 
-  endpoint->AddAllCapabilities ();
-
   while (IsOpen() && !stop) {
 
     calling_state = endpoint->GetCallingState ();
@@ -267,7 +238,7 @@ GMLid::Main ()
     if (c) {
 
       gnomemeeting_threads_enter ();
-      gnomemeeting_dialpad_event (c);
+      gm_main_window_press_dialpad (main_window, c);
       gnomemeeting_threads_leave ();
 
       last_key_press = PTime ();
@@ -279,14 +250,16 @@ GMLid::Main ()
     if (off_hook == TRUE && last_off_hook == FALSE) {
 
       gnomemeeting_threads_enter ();
-      gnomemeeting_log_insert (gw->history_text_view, _("Phone is off hook"));
-      gnomemeeting_statusbar_flash (gw->statusbar, _("Phone is off hook"));
+      gm_history_window_insert (history_window, _("Phone is off hook"));
+      gm_main_window_flash_message (main_window, _("Phone is off hook"));
       gnomemeeting_threads_leave ();
 
 
       /* We answer the call */
+      gnomemeeting_threads_enter ();
       if (calling_state == GMH323EndPoint::Called)
 	GnomeMeeting::Process ()->Connect ();
+      gnomemeeting_threads_leave ();
 
 
       /* We are in standby */
@@ -300,13 +273,14 @@ GMLid::Main ()
 	else
 	  EnableAudio (0, FALSE);
 
+	/* Get the URL from the GUI and call it */
 	gnomemeeting_threads_enter ();
-	url = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry)); 
-	gnomemeeting_threads_leave ();
-
+	url = gm_main_window_get_call_url (main_window);
+	
 	/* Automatically connect */
 	if (url && !GMURL (url).IsEmpty ())
-	  GnomeMeeting::Process ()->Connect ();
+	  GnomeMeeting::Process ()->Connect (url);
+	gnomemeeting_threads_leave ();
       }
     }
 
@@ -315,20 +289,21 @@ GMLid::Main ()
     if (off_hook == FALSE && last_off_hook == TRUE) {
 
       gnomemeeting_threads_enter ();
-      gnomemeeting_log_insert (gw->history_text_view, _("Phone is on hook"));
-      gnomemeeting_statusbar_flash (gw->statusbar, _("Phone is on hook"));
-  
-      /* Remove the current called number */
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry), 
-			  GMURL ().GetDefaultURL ());
+      gm_history_window_insert (history_window, _("Phone is on hook"));
+      gm_main_window_flash_message (main_window, _("Phone is on hook"));
+      gm_main_window_set_call_url (main_window, GMURL ().GetDefaultURL ());
       gnomemeeting_threads_leave ();
 
       RingLine (0, 0);
 
       /* If we are in a call, or calling, we disconnect */
       if (calling_state == GMH323EndPoint::Connected
-	  || calling_state == GMH323EndPoint::Calling)
+	  || calling_state == GMH323EndPoint::Calling) {
+	
+	gnomemeeting_threads_enter ();
 	GnomeMeeting::Process ()->Disconnect ();
+	gnomemeeting_threads_leave ();
+      }
     }
 
 
@@ -342,29 +317,33 @@ GMLid::Main ()
       if ((t.GetSeconds () > 3 && !do_not_connect)
 	  || c == '#') {
 
-	if (calling_state == GMH323EndPoint::Standby)
-	  GnomeMeeting::Process ()->Connect ();
-
+	if (calling_state == GMH323EndPoint::Standby) {
+	  
+	  gnomemeeting_threads_enter ();
+	  url = gm_main_window_get_call_url (main_window);
+	
+	  /* Automatically connect */
+	  if (url && !GMURL (url).IsEmpty ())
+	    GnomeMeeting::Process ()->Connect (url);
+	  gnomemeeting_threads_leave ();
+	}
 	do_not_connect = TRUE;
       }
 
 
-      /* *1 to hold a call */
+      /* *1 to hold the current call */
       if (old_c == '*' && c == '1'
 	  && calling_state == GMH323EndPoint::Connected) {
 
+	call_token = endpoint->GetCurrentCallToken ();
+	is_on_hold = endpoint->IsCallOnHold (call_token);
+	if (endpoint->SetCallOnHold (call_token, !is_on_hold))
+	  is_on_hold = !is_on_hold; /* It worked */
+
+
+	/* Update the GUI */
 	gnomemeeting_threads_enter ();
-	hold_call_cb (NULL, NULL);
-	gnomemeeting_threads_leave ();
-      }
-
-
-      /* *2 to transfer a call */
-      if (old_c == '*' && c == '2'
-	  && calling_state == GMH323EndPoint::Connected) {
-
-	gnomemeeting_threads_enter ();
-	g_idle_add (transfer_callback_cb, NULL);
+	gm_main_window_set_call_hold (GTK_WIDGET (main_window), is_on_hold);
 	gnomemeeting_threads_leave ();
       }
     }
@@ -379,11 +358,17 @@ GMLid::Main ()
     /* We must poll to read the hook state */
     PThread::Sleep (50);
   }
+}
 
-  /* Update the codecs list */
-  gnomemeeting_threads_enter ();
-  gnomemeeting_codecs_list_build (pw->codecs_list_store, FALSE, FALSE);
-  gnomemeeting_threads_leave ();
+
+OpalMediaFormat::List
+GMLid::GetAvailableAudioCapabilities ()
+{
+  OpalMediaFormat::List list;
+
+  list = GetMediaFormats ();
+
+  return list;
 }
 
 
@@ -392,7 +377,7 @@ GMLid::UpdateState (GMH323EndPoint::CallingState i)
 {
   int lid_odt = 0;
 
-  lid_odt = gconf_get_int (AUDIO_DEVICES_KEY "lid_output_device_type");
+  lid_odt = gm_conf_get_int (AUDIO_DEVICES_KEY "lid_output_device_type");
   
   if (IsOpen ()) {
 
@@ -439,12 +424,15 @@ GMLid::UpdateState (GMH323EndPoint::CallingState i)
 
 
 void
-GMLid::SetVolume (int x, int y)
+GMLid::SetVolume (BOOL is_encoding, 
+                  int x)
 {
   if (IsOpen ()) {
 
-    SetPlayVolume (0, x);
-    SetRecordVolume (0, y);
+    if (!is_encoding)
+      SetPlayVolume (0, x);
+    else
+      SetRecordVolume (0, x);
   }
 }
 
